 /*
 * File:        common.h
 * Purpose:     Common defines
 * Copyright:   (C) Pascal Chour
 * Licence:     GPL
 */
#ifndef __INCLUDE_COMMON_H
#define __INCLUDE_COMMON_H

#include <msp430fr6989.h>
#include "stdint.h"
#include "stdbool.h"

/* Define pour programme Geiger - Pascal Chour */

//#define TEST_LCD // gnration de code pour la version en dveloppement => Tests LCD

/* Ports et PIN utiliss */

#define PORT_TX GPIO_PORT_P4 // TX processeur (out)
#define PIN_TX GPIO_PIN2

#define PORT_RX GPIO_PORT_P4 // RX Processeur (in)
#define PIN_RX GPIO_PIN3


#define PORT_CLAVIER GPIO_PORT_P2 // Port Clavier 3x4
#define PIN_CLAVIER1 GPIO_PIN1 // PIN clavier
#define PIN_CLAVIER2 GPIO_PIN2
#define PIN_CLAVIER3 GPIO_PIN3
#define PIN_CLAVIER4 GPIO_PIN4 // PIN clavier sous IT
#define PIN_CLAVIER5 GPIO_PIN5
#define PIN_CLAVIER6 GPIO_PIN6
#define PIN_CLAVIER7 GPIO_PIN7
#define PxIFG_CLAVIER P2IFG
#define PxIV_CLAVIER P2IV_P2IFG7

#define PORT_AL_LED GPIO_PORT_P8 // AL_LED (out)
#define PIN_AL_LED GPIO_PIN4

#define PORT_AL_BUZ GPIO_PORT_P8 // Alarme buzzer (out)
#define PIN_AL_BUZ GPIO_PIN5

#define PORT_EN_LED GPIO_PORT_P8 // EN_LED (out)
#define PIN_EN_LED GPIO_PIN6

#define PORT_EN_BUZ GPIO_PORT_P8 // EN_buzzer (out)
#define PIN_EN_BUZ GPIO_PIN7

#define PORT_SLEEP GPIO_PORT_P3 // Sleep (in)
#define PIN_SLEEP GPIO_PIN2

#define PORT_DETECT1 GPIO_PORT_P3 // DETECT1 (in)
#define PIN_DETECT1 GPIO_PIN7
#define PORT_TBCLK GPIO_PORT_P3 // DETECT1 (in)
#define PIN_TBCLK GPIO_PIN3


#define PORT_BAT_EN GPIO_PORT_P9 // BATen (out)
#define PIN_BAT_EN GPIO_PIN1

#define PORT_BAT_VOLT GPIO_PORT_P9 // Lecture tension batterie (in)
#define PIN_BAT_VOLT GPIO_PIN0

#define PORT_LED1 GPIO_PORT_P1 // LED1 de la carte
#define PIN_LED1 GPIO_PIN0

#define PORT_LED2 GPIO_PORT_P9 // LED2 de la carte
#define PIN_LED2 GPIO_PIN7



/* variables d'tat du systme */

#define periode_affichage (5)           // Periode pour rafraichissement LCD et calculs en secondes. Typ : 5 secondes
#define periode_par_minute (60/periode_affichage)    // Nombre de periodes par minute.
#define nb_minutes_par_24h  (1440)      // nb de minutes en 24H
#define alarme_defaut (50)              // valeur de seuil pour alarme par dfaut
#define NbMoyHeures (8760)              // Nombre d'heures mmorises dans le buffer circulaire

#define INACTIVITE_MAX (20)             // pour sortie du mode dialogue si inactivit (en secondes)
#define ALARME_MAX     (10)             // dure de l'alarme en secondes

/* seuil Tensions batteries. Tension centivolts */

#define V_BAT_SEG_1 (345)               // Batterie 1 segment
#define V_BAT_SEG_2 (355)               // Batterie 2 segment
#define V_BAT_SEG_3 (365)               // Batterie 3 segment

#define BUF_MAX (30)                    // taille max d'un buffer. A priori, le max est ID=XXXX,XXXX,XXXX,XXXX
#define MaxPileES (4)                   // Taille max pile pour commandes entres sorties

typedef enum {TOUCHE_RIEN, TOUCHE_VALIDE, TOUCHE_PLUS, TOUCHE_MOINS, TOUCHE_BAT, TOUCHE_SEUIL, TOUCHE_ALLED,
              TOUCHE_ALBUZ, TOUCHE_ENLED, TOUCHE_ENBUZ, TOUCHE_MAX, TOUCHE_MOY, TOUCHE_CPM} T_TOUCHE ;

typedef struct
{
  bool a_traiter;       // vrai si quelque chose  traiter
  uint8_t  index;       // index courant du buffer
  char buf[BUF_MAX];    // buffer  traiter
} tESbuf;
  
typedef struct // mmorisation des moyennes par heures
{
    uint16_t Buf[NbMoyHeures]; // contient les dernires NbMoyHeures moyennes par heures
    int16_t Index; // Index courant pour stockage dans MoyHeures. Doit tre initialis  -1 au dmarrage
    bool Plein; // Indique si le buffer circulaire est plein.
    int16_t IndexES; // Index courant de la valeur  mettre dans MoyHeures suite  une commande Download
} TMoyHeures;

// PileES et ses fonctions associes (PUSH, POP...) permettent de mmoriser les
// actions  traiter au niveau E/S. On limite  MaxPilesES+1 le nombre d'actions.
// CmdDL1 => mettre le premier DL=xxxx ou NODATA
// CmdDL2 => mettre les DL=xxxx suivants jusqu' END
// CmdDL3 => mettre DL=END
// CmdCM => mettre un CM=xxxx
// CmdCF => mettre CF=xxxx
// cmdVP => mettre VP=... (N) de version, type compteur, n de srie)
// CmdPC => mettre paramtres compteurs
enum {CmdDL1, CmdDL2, CmdDL3, CmdVide, CmdCM, CmdCF, CmdVP, CmdPC, CmdGD};

typedef struct {
    int16_t Index;
    char Commande[MaxPileES+1];
} TPileES;

// Variables globales externes
extern uint32_t countMoyenneHeure[25];         // nb d'heures +1 par jours (main.c)
extern TMoyHeures MoyHeures;
extern uint16_t NbMinutes24h;                  // nb de minutes coules depuis le dbut du comptage limit  24h00 (main.c)
extern uint16_t NbJours;                       // Nb de jours depuis que la mise en marche du compteur
extern uint16_t CountMax;                      // Valeur max courante. 0 = pas initialise
extern int8_t Sens;                            // Sens de rotation et nombre d'impulsion
extern uint8_t CompteInactivite;               // compteur d'inactivit pour sortir du menu (en secondes)
extern bool MemoireAlarme;
extern uint8_t CompteAlarme;
extern uint32_t CountParMinute;                // Pour compter le nombre d'impulsions par minutes.
extern uint16_t Vbat;                          // dernire tension batterie mesure
extern bool AlarmeLed;                         // vrai si on fait clignoter la LED si alarme
extern bool AlarmeBuz;                         // vrai si on active le buzzer si alarme
extern bool EnableLed;                         // Vrai si on active le TIC par LED
extern bool EnableBuz;                         // vrai si on active le TIC par buzzer
extern uint16_t SeuilAlarme;                   // Seuil  partir duquel on gnre une alarme (main.c)
extern bool VerrouilleClavier;                 // Verrouillage/dverrouillage clavier

extern const char T_Invite[];
extern const uint8_t Posit[];


// Variables modifiables sous IT
volatile extern bool BasculeActivite;                   // pour faire clignoter l'indicateur d'activit
extern volatile uint8_t ValClavier;                     // Valeur lue au clavier (interruption)
extern volatile uint8_t ETAT_ALARME;
extern volatile char C_TraitementPeriodique;            // actions  raliser (compteur) : calcul et autres traitements priodiques  traiter


#endif // __INCLUDE_COMMON_H 
