/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/********************************************************************************
 * Fichier : main.c
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019 - 19/07/2020
 * (c) Pascal Chour
 ********************************************************************************/

#include <hal_LCD.h>
#include "main.h"
#include "common.h"
#include "menu.h"
#include "utilitaires.h"
#include "driverlib/MSP430FR5xx_6xx/driverlib.h"


/* Variables d'tat du systme (globales) */

uint16_t countMoyenneMin[periode_par_minute]; // pour mmorisation de moyennes sur 1 mn glissante. Modifiable par IT.
uint32_t countMoyenneHeure[25];        // nb d'heures +1 par jours. Modifiable par IT.
#pragma PERSISTENT(MoyHeures);
TMoyHeures MoyHeures = {0};
uint16_t NbJours = 1;                  // Pour mmorisation du nombre de jours de fonctionnement du compteur depuis son dmarrage.
uint16_t NbMinutes24h = 0;             // nb de minutes coules depuis le dbut du comptage limit  24h00..
uint8_t NbMinutes = 0;                 // nb de minutes pour le passage des heures.
uint8_t CptPeriode = 0;                // pour comptage priode affichage et calcul. Evite une opration modulo.
uint8_t NbTiPmn = 0;                   // index courant dans tableau comptage par minutes.
uint8_t NbTiPh = 0;                    // index courant dans tableau comptage par heures.
uint8_t CompteAlarme = 0;              // Pour compter la signalisation de l'alarme courante (en secondes).
uint8_t CompteInactivite = 0;          // compteur d'inactivit pour sortir du menu (en secondes).
uint32_t CountParMinute = 0;           // Pour compter le nombre d'impulsions par minutes.
uint16_t CountMax = 0;                 // Valeur max courante. 0 = pas initialise.
uint16_t CountLast = 0;                // dernire valeur du compteur d'impulsion.
bool MemoireAlarme = false;            // Pour indiquer qu'on a une alarme  valider

// variables globales modifiables par IT
volatile bool BasculeActivite = false; // pour faire clignoter l'indicateur d'activit
volatile char C_TraitementPeriodique = 0;       // actions  raliser (compteur) : calcul
volatile char C_Clavier = 0;                    // actions  raliser (compteur) : clavier
volatile uint16_t CountCurrent;                 // Pour lecture du compteur d'impulsion


/*
 * main.c
 */
int main(void) {
uint16_t i;
/*
 * dans cette section, on ne doit pas avoir besoin des interruptions pour fonctionner !
 */

    // Au dmarrage, les interruptions sont supposes masques (bit GIE de SR  0).
    // Stop Watchdog timer
 WDT_A_hold(__MSP430_BASEADDRESS_WDT_A__);     // Stop WDT

/*
 * Dans cette section, on ne peut appeler que des routines qui n'utilisent pas de timers
 * ou d'autres interruptions.
 */
    InitGenerale(); // doit tre appel en premier. Initialisation par dfaut des GPIO, horloges, certaines variables, etc.
    LCD_init(); // initialisation de l'afficheur LCD

    // principe de comptage : un tableau comportant "periode_par_minute" positions
    // est utilis pour calculer une moyenne glissante. Ex : si une priode de comptage est
    // 10s, alors, il y a 6 priodes de comptage par minute. Ce tableau est initialis
    //  0 au dmarrage.
    // Il faut attendre une minute pour que la moyenne de comptage soit
    // correcte.
    for (i = 0; i < periode_par_minute;i++) {countMoyenneMin[i] = 0;}

    // on initialise les tableaux pour comptage par heure sur 24H
    for (i = 0; i < 25; i++) {countMoyenneHeure[i] = 0;}

    // Buffer des heures mmorises
    for (i = 0; i < NbMoyHeures; i++) {MoyHeures.Buf[i] = 0;}
    MoyHeures.Index = -1;
    MoyHeures.Plein = false;
    MoyHeures.IndexES = -1;

    __enable_interrupt(); // Dmasquage des interruptions dans le Status Register (bit GIE de SR)

/*
 * A partir de cettes section, toutes les IT utiles sont actives. On peut donc utiliser l'afficheur LCD, la roue codeuse ou le poussoir, etc.
 * Le RTC tant actif, on voit le symbole d'activit clignoter sur l'afficheur LCD
 */
    LCD_clear();
    displayScrollText(T_Invite); // Affichage d'un petit texte d'introduction
    LCD_displayLevel(0, true, false); // Affichage des barres droites et gauche de l'indicateur de niveau.

#ifdef TEST_LCD             // Squence de test du LCD
    for (i=0; i <= 6; i++) {
        LCD_displayLevel(i,true, true);
    }
    for (i=6; i != 0; i--) {
        LCD_displayLevel(i-1,true, true);
    }
    LCD_displayLevel(0, false, false);
    LCD_displayTmr(true, true);
    LCD_displayTmr(false, false);
    LCD_displayAnt(true, true);
    LCD_displayAnt(false, false);
    LCD_displayAlarm(true, true);
    LCD_displayAlarm(false, false);
    LCD_displayHeart(true, true);
    LCD_displayHeart(false, false);
    LCD_clear();            // effacement de l'afficheur
#endif
    TensionBatterie(); // met  jour VBat
    InitialiseES();

    while(1) {
        InitialiseES(); // Teste si on doit initialiser ou clore les E/S
        TraitementPeriodique();
        MenuAutomate();
        __low_power_mode_3 ();
    }
}


/*
 * RTC Interrupt Service Routine
 * Rveil toutes les secondes. Effectue la mesure du comptage et l'affichage sur l'cran LCD.
 * Met  jour l'indicateur d'alerte si besoin. Fait clignoter l'indicateur d'activit
 */
//#pragma vector = RTC_VECTOR
//__interrupt void RTC_ISR(void)
#pragma vector=WDT_VECTOR
__interrupt void watchdog_isr( void )
{
    // Si P3.7 cbl, alors, on peut lire TB0CCR3. Sinon, il faut lire TB0R
    CountCurrent = TB0CCR3; /* lecture du compteur courant */
    BasculeActivite = !BasculeActivite;
    if (BasculeActivite) {LCDMEM[2] |=  0x04;} else {LCDMEM[2] &= 0xFB;} // LCD_displayHeart(BasculeActivite, false);
    C_TraitementPeriodique = C_TraitementPeriodique+1;
    __low_power_mode_off_on_exit();
}
/*
 * Effectue les calculs et les traitements priodiques (par seconde ou par priode)
 */

void TraitementPeriodique()
{
    uint16_t i;
    uint32_t Bid;

    if (C_TraitementPeriodique)
    {
        C_TraitementPeriodique = C_TraitementPeriodique-1;
        switch (CptPeriode)
        {
            case 0 :   // Toutes les "periodes_affichage"
                countMoyenneMin[NbTiPmn] = 0;
                if (CountCurrent < CountLast) { /* Petit traitement si le compteur a dbord. On suppose qu'il ne peut y avoir plus de FFFF impulsions entre deux calculs */
                    countMoyenneMin[NbTiPmn] = (0xFFFF-CountLast)+CountCurrent;
                } else {
                    countMoyenneMin[NbTiPmn] = CountCurrent-CountLast;
                }
                CountLast = CountCurrent; // On conserve la valeur pour le coup d'aprs
                // Nb de comptage par minutes. Il faut attendre une minute pour que le comptage soit valide.
                CountParMinute = 0;
                for (i = 0; i < periode_par_minute; i++) {
                    CountParMinute = CountParMinute+countMoyenneMin[i];
                }

                // Traitement du max et de l'alarme
                if (CountParMinute > CountMax) {CountMax = CountParMinute;}
                if (CountParMinute > SeuilAlarme) {
                    if (CompteAlarme == 0) {
                        MemoireAlarme = true;           // Pour signaler l'alarme sur l'afficheur LCD
                        LCD_clignoteTxt(true);
                        LCD_displayAlarm(true, true);
                        LCDMEM[2] |= 0x01;              // Symbole alarme
                        ETAT_ALARME = E_ALARME0;        // Initialisation de l'automate de signalement d'alarme
                        if (!(TA0CTL & (MC0 | MC1))) {  // Si le timer est  arrt
                            TA0CCTL0 = CCIE;            // Autorise interruption sur CCIFG : Interruption si le compteur atteind une certaine valeur.
                            TA0CCR0 = PERIODE_1ms;       // Valeur  comparer pour interruption. Correspond environ  1ms
                            TA0R = 0;                   // Compteur dmarre  0
                            TA0CTL = TASSEL_1 + MC_2;   // TASSEL_1 => utilisation de ACLK (32768Hz), MC_1 => compare mode jusqu' TB0CCR0
                        }
                    }
                    CompteAlarme = ALARME_MAX;           // Pour commencer  compter la dure de signalement de l'alarme LED ou Buzzer
                }
                NbTiPmn = NbTiPmn+1;

                if (NbTiPmn >= periode_par_minute) {// Traitement toutes les minutes
                    NbTiPmn = 0;

                    // On suppose qu'il ne peut pas y avoir de dbordement car compteur 32 bits ! Pour mmoire, 1000 coups par seconde * 60 * 60 = 3600000.
                    countMoyenneHeure[NbTiPh] += CountParMinute;
                    if (CountParMinute >= 0xFFFF) {CountParMinute = 0xFFFF;}
                    PileESPUSH(CmdCM);

                    if (NbMinutes24h <= nb_minutes_par_24h) {NbMinutes24h = NbMinutes24h+1;} // utilis pour le calcul de la moyenne

                    NbMinutes = NbMinutes+1; // Et par ailleurs, on a un compteur de minutes pour faire passer les heures
                    if (NbMinutes > 60) {
                      NbMinutes = 0;
                      MoyHeures.Index = MoyHeures.Index+1;
                      if (MoyHeures.Index >= NbMoyHeures) {
                          MoyHeures.Index = 0;
                          MoyHeures.Plein = true; // le buffer circulaire est plein
                      }
                      Bid = countMoyenneHeure[NbTiPh]/60;
                      if (Bid > 0xFFFF) {
                          MoyHeures.Buf[MoyHeures.Index] = 0xFFFF;
                      } else {
                          MoyHeures.Buf[MoyHeures.Index] = Bid;
                      } // moyenne des CPM par heures
                      NbTiPh = NbTiPh+1;
                      if (NbTiPh > 24) {
                          NbTiPh = 0;
                          NbJours = NbJours+1;
                          TensionBatterie(); // met  jour VBat
                      }
                      countMoyenneHeure[NbTiPh] = 0;
                    }
                }
                // traitement de l'affichage
                // si pas d'affichage du menu, alors, on affiche le comptage
                if (CompteInactivite == 0)  {LCD_writeIntRight(CountParMinute);}

                /* affichage bargraph chelle log */
                if (CountParMinute == 0) {
                    LCD_displayLevel(0,true, false); // dsactive clignotement sur bargraph
                } else {
                  if (CountParMinute < 20) {
                      LCD_displayLevel(1,true, false);
                  } else {
                    if (CountParMinute < 40) {
                        LCD_displayLevel(2,true, false);
                    } else {
                      if (CountParMinute < 80) {
                          LCD_displayLevel(3,true, false);
                      } else {
                        if (CountParMinute < 160) {
                            LCD_displayLevel(4,true, true); // active clignotement sur bargraph
                        } else {
                          if (CountParMinute < 320) {
                              LCD_displayLevel(5,true, true);
                          } else {
                              LCD_displayLevel(6,true, true);
                          }
                        }
                      }
                    }
                  }
                }

            default :
                if (CompteInactivite > 0) {
                    CompteInactivite--;
                    if (CompteInactivite == 0) {
                        LCD_ClearText();
                        LCD_writeIntRight(CountParMinute);
                    }
                }
                 if (CompteAlarme > 0) {CompteAlarme--;}
            break;
        }
        if (CptPeriode >= periode_affichage) {CptPeriode = 0;} else {CptPeriode = CptPeriode+1;}   // mise   jour du compteur de seconde. Remis  zro toutes les priodes de temps
    }

}
