/*
 * Fichier : menu.c
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019
 * (c) Pascal Chour
 */

#include <hal_LCD.h>
#include "common.h"
#include "menu.h"
#include "utilitaires.h"



bool    AlarmeLed = false;                  // vrai si on fait clignoter la LED si alarme */
bool    AlarmeBuz = false;                  // vrai si on active le buzzer si alarme */
bool    EnableLed = false;                  // Vrai si on active le TIC par LED
bool    EnableBuz = false;                  // vrai si on active le TIC par buzzer
bool    VerrouilleClavier = false;          // Clavier dverrouill par dfaut
uint8_t DernierValClavier = TOUCHE_RIEN;
volatile uint8_t ValClavier = 0;            // Valeur lue au clavier
uint16_t SeuilAlarme = alarme_defaut;       // Seuil  partir duquel on gnre une alarme

 /*
 Gre le menu du compteur.
 Auteur : Pascal Chour

*/


void MenuAutomate()
{
  uint8_t ValClavier;
  uint16_t i;
  uint32_t AffMoy;    // Moyenne courante


  if (LireCar(&ValClavier))
  {

      if ((VerrouilleClavier) && (ValClavier != TOUCHE_CPM) && (ValClavier != TOUCHE_VALIDE)) {return;}

      switch (ValClavier)
      {
        case TOUCHE_CPM: // utilis uniquement pour verrouillage/dverrouillage
            CompteInactivite = 3;
            break;

        case TOUCHE_MAX:
            LCD_writeTxtLeft("MAX", true);
            LCD_writeIntRight (CountMax);
            MemoireAlarme = false;
            LCD_displayAlarm(false, false);
            CompteInactivite = 3;
            CountMax = CountParMinute; // Rinitialise CountMax  la valeur  courante
        break;
        case TOUCHE_VALIDE:
            if ((DernierValClavier == TOUCHE_CPM)  && (CompteInactivite > 0)) {
                VerrouilleClavier = !VerrouilleClavier;
                if (VerrouilleClavier) {LCD_writeTxtLeft("LOCK", true);} else {LCD_writeTxtLeft("UNLOCK", false);}
                CompteInactivite = 3;
                return;
            }
            if (VerrouilleClavier) {return;}
            if (CompteInactivite > 0) {
                CompteInactivite = 0;
                LCD_ClearText();
                return;
            }
        case TOUCHE_MOY:
            LCD_writeTxtLeft("MOY", true);
            AffMoy = 0;
            for (i = 0; i < 25;i++) {AffMoy = countMoyenneHeure[i]+AffMoy;}
            if (NbMinutes24h > 0) {AffMoy = AffMoy/NbMinutes24h;} else {AffMoy = 0;}
            LCD_writeIntRight (AffMoy);
            CompteInactivite = 3;
        break;
        case TOUCHE_BAT:
            LCD_writeTxtLeft("BAT", true);
            TensionBatterie(); // met  jour VBat
            LCD_writeIntRight (Vbat);
            CompteInactivite = 3;
        break;

        case TOUCHE_ENLED:
            EnableLed = !EnableLed;
            if (EnableLed) {LCD_writeTxtLeft("SLED I", false);} else {LCD_writeTxtLeft("SLED O", false);}
            CompteInactivite = 3;
        break;

        case TOUCHE_ENBUZ:
            EnableBuz = !EnableBuz;
            if (EnableBuz) {LCD_writeTxtLeft("SBUZ I", false);} else {LCD_writeTxtLeft("SBUZ O", false);}
            CompteInactivite = 3;
        break;

        case TOUCHE_ALLED:
            AlarmeLed = !AlarmeLed;
            if (AlarmeLed) {LCD_writeTxtLeft("ALED I", false);} else {LCD_writeTxtLeft("ALED O", false);}
            CompteInactivite = 3;
        break;

        case TOUCHE_ALBUZ:
            AlarmeBuz = !AlarmeBuz;
            if (AlarmeBuz) {LCD_writeTxtLeft("ABUZ I", false);} else {LCD_writeTxtLeft("ABUZ O", false);}
            CompteInactivite = 3;
        break;
        //  Pour le moment, les touches +/-  ne servent qu'aux seuils
        case TOUCHE_MOINS:
        case TOUCHE_PLUS:
            if (ValClavier == TOUCHE_PLUS) {
                if (SeuilAlarme < 9999) {SeuilAlarme = SeuilAlarme+1;}
            } else if (ValClavier == TOUCHE_MOINS) {
                if (SeuilAlarme > 0) {SeuilAlarme = SeuilAlarme-1;}
            }

        case TOUCHE_SEUIL:
            LCD_writeTxtLeft("AL", true);
            LCD_writeIntRight (SeuilAlarme);
            CompteInactivite = 3;
        break;

      }
      if (!VerrouilleClavier) {
          // Eteint les alarmes suite  saisie d'un caractre. Comme on ne sait pas dans quel tat sont les signaux Enable, on les rinitialise systmatiquement
          GPIO_setOutputLowOnPin(PORT_AL_BUZ, PIN_AL_BUZ);
          GPIO_setOutputLowOnPin(PORT_AL_LED, PIN_AL_LED);
          if (EnableLed) {GPIO_setOutputHighOnPin(PORT_EN_LED, PIN_EN_LED);} else {GPIO_setOutputLowOnPin(PORT_EN_LED, PIN_EN_LED);}
          if (EnableBuz) {GPIO_setOutputHighOnPin(PORT_EN_BUZ, PIN_EN_BUZ);} else {GPIO_setOutputLowOnPin(PORT_EN_BUZ, PIN_EN_BUZ);}
          CompteAlarme = 0;
      }
      DernierValClavier = ValClavier;
  }
}



