/*
 * Fichier : utilitaires.c
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019
 * (c) Pascal Chour
 */

#include <hal_LCD.h>
#include "utilitaires.h"
#include "stdbool.h"
#include "string.h"
#include "common.h"
#include "menu.h"



TPileES PileES;

// Backup Memory variables to track states through LPM3.5
// Dans le cas prsent, utilis pour stocker des variables de compteurs ou d'tat
volatile uint8_t  ClavierRebond = 0;            // Anti-rebond clacier
volatile uint8_t ETAT_ALARME = E_ALARME0;       // Etat automate pour gestion des signalisations d'alarmes
volatile uint8_t KB_L = 0;                      // Ligne lue sur clavier
volatile uint8_t KB_C = 0;                      // Colonne lue sur clavier
volatile uint16_t ALARME_TIMER = 0;             // Pour frquence du bip
volatile uint16_t CPTSleep = 0;                 // Compteur pour mode sleep. Max 65535 ms
volatile uint8_t CarClavier = 0;                // dernier caractre tap au clavier
uint16_t Vbat;                                  // dernire tension lue
bool ESInitialise = false;                      // Vrai si E/S initialises. Faux sinon
volatile tESbuf recept;                         // Buffer rception E/S asynchrone
tESbuf emet;                                    // buffer mission E/S asynchrone
const char T_Invite[] = "VP=1.0,PCGM9,1";       // texte d'invite version X, PCGM8, n serie = Y
const char T_FacteurConversion[] = "CF=0.0057"; // facteur de conversion pour SBM20 renvoy suite  commande "C" reue.
const char T_NoData[] = "NODATA";               // Commande Dwonload, NoData
const char T_End[] = "END";                     // Commande de fin de download
const char T_DL[]  = "DL=";
const uint8_t keys[4][3] = {
   {TOUCHE_ALLED,TOUCHE_BAT,TOUCHE_PLUS},
   {TOUCHE_ALBUZ,TOUCHE_MAX,TOUCHE_SEUIL},
   {TOUCHE_ENLED,TOUCHE_MOY,TOUCHE_MOINS},
   {TOUCHE_ENBUZ,TOUCHE_VALIDE,TOUCHE_CPM}
};




/*
 * GPIO Initialization
 */
void InitGenerale()
{
    // Initialisation des GPIO au niveau bas en sortie pour limiter la consommation. Les ports ABCDE sont initialiss dans le module hal_LCD
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6);
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P5, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P6, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P7, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P8, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P9, GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

    GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P3, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6);
    GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P5, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P6, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P7, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P8, GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
    GPIO_setAsOutputPin(GPIO_PORT_P9, GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);

    // Init Clavier

    P2DIR   = 0x0F; // P2.7..4 input P2.3..0 output
    P2REN   = 0xFF; // P2.7..0 REN enabled
    P2OUT   = 0xF0; // PullUp sur P2.7..4, Low sur P3..0
    P2IES   = 0xF0; // P2.7..4 IT on High to Low transition
    P2IE    = 0xF0; // P2.7..4 IT Enabled, P2.3..1 IT Disabled


    // Entre voltage
    GPIO_setAsPeripheralModuleFunctionInputPin(PORT_BAT_VOLT,PIN_BAT_VOLT,GPIO_TERNARY_MODULE_FUNCTION);


    // AL_BUZZ off - Dj initialis dans les initialisations par dfaut
    // AL_LED off - Dj initialis dans les initialisations par dfaut
    // DC off - Dj initialis dans les initialisations par dfaut
    // Mesure batterie enable - off - Dj initialis dans les initialisations par dfaut

    // Sleep FTDI32
    GPIO_setAsInputPinWithPullDownResistor (PORT_SLEEP, PIN_SLEEP);

    // DETECT1
    GPIO_setAsPeripheralModuleFunctionInputPin(PORT_DETECT1,PIN_DETECT1,GPIO_SECONDARY_MODULE_FUNCTION);
    GPIO_setAsPeripheralModuleFunctionInputPin(PORT_TBCLK,PIN_TBCLK,GPIO_TERNARY_MODULE_FUNCTION);

    // Initialisation compteur d'impulsion TB0 sur P3.7
    TB0CTL = TBCLR;
    TB0CTL = 0x0020; // TBCLK+16 bits +diviseur = 1 + Continuous mode + capture mode + pas d'interruptions
    TB0CCTL3 = 0x5100; // CCI3B + Capture front montant + asynchrone + Pas d'interruption.

    // Initialisation des ports d'E/S
    GPIO_setAsInputPinWithPullDownResistor(PORT_RX, PIN_RX); // RX en input, TX dj initialis en output  low. Primary function slectionn uniquement si besoin

    // PJ4/J5 servent au quartz
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_PJ, GPIO_PIN4 + GPIO_PIN5, GPIO_PRIMARY_MODULE_FUNCTION );

    // Autorise les modifications du PMM pour la suite (utile pour les ADC).
    PMMCTL0_H = PMMPW_H;                    // Unlock the PMM registers

    // Disable the GPIO power-on default high-impedance mode
    // to activate previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    // Set DCO frequency to default 8MHz
    CS_setDCOFreq(CS_DCORSEL_0, CS_DCOFSEL_6);  // DCORSEL = 0 et DCORSEL = 6 => 8MHz

    // Configure MCLK and SMCLK to default 2MHz
    CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

    // Intializes the XT1 crystal oscillator
    CS_turnOnLFXT(CS_LFXT_DRIVE_3);

    WDTCTL = WDT_ADLY_1000; // Password, arrt WDT, Horloge = XCLK, Interval timer mode, toute les secondes WDT_ADLY dfinit dans msp430fr6889.h
    SFRIE1 |= WDTIE;
    WDT_A_start(WDT_A_BASE);
    GPIO_clearInterrupt(PORT_CLAVIER, PIN_CLAVIER7 || PIN_CLAVIER6 || PIN_CLAVIER5 || PIN_CLAVIER4);

}

/*
 * Donne le dernier caractre tap. 0 si pas de caractre tap et Faux. Le caractre et vrai si un caractre est disponible
 */
bool LireCar(uint8_t *c)
{
    *c = CarClavier;
    if (CarClavier) {
        CarClavier = 0;
        return true;
    } else {
        return false;
    }
}


/* Donne le max entre 2 uint16
 * En entre : A et B
 * En sortie : A si A >= B ou B si B > A
 */
uint16_t max16(uint16_t a,uint16_t b)
{
    if(a>=b) return a; else return b;
}

/*
 * Filtre les espaces
 */

void lecteur_sauterblancs(void)
{
  while ((recept.buf[recept.index] == ' ') && (recept.index < BUF_MAX) && (recept.buf[recept.index] != 0x00)) {
    recept.index++;
  }
}

/*
 * Donne le caractre courant dans le lecteur en liminant les blancs
 */

char lecteur_carcourantetinc(void)
{
  char c;
  c = recept.buf[recept.index];
  if ((recept.index < BUF_MAX) && (recept.buf[recept.index] != 0x00)) {recept.index++;}
  lecteur_sauterblancs();
  return(c);
}

/*
initialise l'analyseur lexical
*/

void lecteur_init(void)
{
  recept.index = 0;
  lecteur_sauterblancs();
}


/*
 * Renvoie vrai si on trouve s dans recept.buf  la position courante de recept.index
*/

bool lecteur_ident(const char *s)
{
  int j;
  int l;
  lecteur_sauterblancs();
  j = 0;
  l = strlen(s);
  while ((recept.buf[recept.index+j] >= 'A') && (recept.buf[recept.index+j] <= 'Z')) {j++;}
  /* en sortie, j = longueur d'un "ident" dans le buffer */
  if (j == l) {
    j = 0;
    while ((recept.buf[recept.index+j] == s[j]) && (j < l)) {
      j++;
    }
    if (j == l) {
      recept.index = recept.index+j;
    }
  }
  return (j == l);
}

/*
lit un entier limit  9999 et rend le rsultat dans nb.
La fonction rend vrai si un entier a t lu.
Si un entier n' pas t lu, la fonction rend faux et
nb est inchang.
*/

bool lecteur_int(unsigned int *nb)
{
  bool nb_valide;
  unsigned int nb_save;
  lecteur_sauterblancs();
  *nb = 0;
  nb_valide = false;
  nb_save = *nb;
  while ((recept.buf[recept.index] >= '0') && (recept.buf[recept.index] <= '9') && (*nb <= 1000)) {
    *nb = *nb*10+(recept.buf[recept.index] & 0x0f);
    recept.index++;
    nb_valide = true;
  }
  if (!nb_valide) {*nb = nb_save;}
  return(nb_valide);
}


/*
 * Formatte un uint16 en ASCII cadr  droite sur 5 caractres max termin par un null. La valeur ASCII est
 * concatne  T1 qui doit tre suffisamment grand et initialis !
 * En entre :  T, chaine de caractre en prfixe de la valeur
 *              Valeur : valeur  convertir
 * En sortie : T = T + Valeur en ASCII.
 */

void Uint16Ascii(uint16_t Valeur, char T1[]) {
    int16_t j;
    char T2[6];
    if (Valeur == 0) {
        strcat(T1,"0\x0");
    } else {
        T2[5] = '\x0';
        j = 4;
        while ((Valeur > 0) && (j >= 0)) {
            T2[j] = Valeur - ((Valeur/10)*10) + 0x30;
            Valeur = Valeur/10;
            j = j-1;
        }
        strcat(T1,&T2[j+1]);
    }
}


/*
 * Suspend le programme pendant Nb x 1ms
 */
void SleepMS(uint16_t Nb)
{
   CPTSleep = Nb;
   __disable_interrupt();
   if (!(TA0CTL & (MC0 | MC1))) {  // Si le timer est  arrt
       TA0CCTL0 = CCIE;            // Autorise interruption sur CCIFG : Interruption si le compteur atteint une certaine valeur.
       TA0CCR0 = PERIODE_1ms;      // Valeur  comparer pour interruption. Correspond environ  1ms
       TA0R = 0;                   // Compteur dmarre  0
       TA0CTL = TASSEL_1 + MC_2;   // TASSEL_1 => utilisation de ACLK (32768Hz), MC_1 => compare mode jusqu' TB0CCR0
   }
   __enable_interrupt();
    while (CPTSleep > 0) {__low_power_mode_3 ();}
}



/*
 * PORT2 Interrupt Service Routine : Clavier
 * Initialise si besoin le timer A0 pour le retournement ligne x colonne
 */
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
    LPM3_EXIT;
    P2IFG   = 0;       // Reset interruption ligne
    if (!(CarClavier)) { // Caractre non lu ? Si oui, on sort. Fait galement office d'antirebond
        KB_L = P2IN & 0xF0; // Lecture ligne
        if (KB_L != 0xF0) { // Rien lu, bizarre... On sort
            P2DIR = 0xF0; // Inversion Input/output entre P2.7..4 et P2.3..0
            P2OUT = KB_L + 0x0F; // P2.3..0 Input avec PullUp, P7..4 output low.
            ClavierRebond = KB_TEMPO;
            if (!(TA0CTL & (MC0 | MC1))) {  // Si le timer est  arrt
                TA0CCTL0 = CCIE;            // Autorise interruption sur CCIFG : Interruption si le compteur atteint une certaine valeur.
                TA0CCR0 = PERIODE_1ms;       // Valeur  comparer pour interruption. Correspond environ  1ms
                TA0R = 0;                   // Compteur dmarre  0
                TA0CTL = TASSEL_1 + MC_2;   // TASSEL_1 => utilisation de ACLK (32768Hz), MC_1 => compare mode jusqu' TB0CCR0
            }
        }
    }
}


/*
 * Timer A0 Interrupt Service Routine : Utilis pour grer les timers (bouton poussoir ou autres)
 */

#pragma vector = TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR (void)
{
    LPM3_EXIT;
    TA0CCTL0 &= ~COV;
    TA0CTL &= ~TAIFG; // Acquittement interruption
    TA0CCR0 += PERIODE_1ms;       // Valeur  comparer pour interruption. Correspond environ  1ms
    // Pour les LED, un flash. Pour le buzzer, 2 bip conscutifs.

    if ((CompteAlarme) && ((AlarmeLed) || (AlarmeBuz))) { // utilis pour indiquer qu'il faut faire bipper ou flasher le compteur. L'tat de l'automate de gestion du clignotement doit avoir t initialis
        // Shift by 8 if port is even (upper 8-bits)
        switch (ETAT_ALARME) {
            case E_ALARME0:
                ALARME_TIMER = 0;
            case E_ALARME1:
                if (!(ALARME_TIMER)) {
                    if (AlarmeLed) {
                        GPIO_setOutputHighOnPin(PORT_AL_LED, PIN_AL_LED); // Signalisation alarme par LED. AlarmeLed indique si l'option est active
                        GPIO_setOutputHighOnPin(PORT_EN_LED, PIN_EN_LED);
                    }
                    if (AlarmeBuz) {
                        GPIO_setOutputHighOnPin(PORT_AL_BUZ, PIN_AL_BUZ); // Signalisation alarme par BUZZER. AlarmeBuz indique si L'option est active
                        GPIO_setOutputHighOnPin(PORT_EN_BUZ, PIN_EN_BUZ);
                    }
                    ALARME_TIMER = DUREE_SIGNAL_ALARME;
                    ETAT_ALARME = E_ALARME2;
                }
                break;
            case E_ALARME2:
                if (!(ALARME_TIMER)) {
                    if (AlarmeLed) {
                        GPIO_setOutputLowOnPin(PORT_AL_LED, PIN_AL_LED); // Signalisation alarme par LED. AlarmeLed indique si l'option est active
                        GPIO_setOutputLowOnPin(PORT_EN_LED, PIN_EN_LED);
                    }
                    if (AlarmeBuz) {
                        GPIO_setOutputLowOnPin(PORT_AL_BUZ, PIN_AL_BUZ); // Signalisation alarme par BUZZER. AlarmeBuz indique si L'option est active
                        GPIO_setOutputLowOnPin(PORT_EN_BUZ, PIN_EN_BUZ);
                    }
                    ALARME_TIMER = DELAI_BUZ;
                    ETAT_ALARME = E_ALARME3;
                }
                break;
            case E_ALARME3:
                if (!(ALARME_TIMER)) {
                    if (AlarmeBuz) {
                        GPIO_setOutputHighOnPin(PORT_AL_BUZ, PIN_AL_BUZ); // Signalisation alarme par BUZZER. AlarmeBuz indique si L'option est active
                        GPIO_setOutputHighOnPin(PORT_EN_BUZ, PIN_EN_BUZ);
                    }
                    ALARME_TIMER = DUREE_SIGNAL_ALARME;
                    ETAT_ALARME = E_ALARME4;
                }
                break;
            case E_ALARME4:
                if (!(ALARME_TIMER)) {
                    if (AlarmeBuz) {
                        GPIO_setOutputLowOnPin(PORT_AL_BUZ, PIN_AL_BUZ); // Signalisation alarme par BUZZER. AlarmeBuz indique si L'option est active
                        GPIO_setOutputLowOnPin(PORT_EN_BUZ, PIN_EN_BUZ);
                    }
                    ALARME_TIMER = PERIODE_LED_BUZ-DUREE_SIGNAL_ALARME-DELAI_BUZ;
                    ETAT_ALARME = E_ALARME1;
                }
                break;
        }
        if (ALARME_TIMER) {ALARME_TIMER = ALARME_TIMER-1;}
    }

    if (ClavierRebond) // tempo retournement ligne x colonne chue ?
    {
        ClavierRebond = ClavierRebond-1; // Non, on dcrmente
        if (!(ClavierRebond)) { // Si chue, alors
            KB_C = (P2IN & 0x0F) >> 1; // On lit la valeur sur P2.3..0 = colonne
            P2DIR = 0x0F; // On remet les ports dans leur tat initial pour le prochain coup
            P2OUT   = 0xF0;
            if (KB_L == 0x70) {KB_L = 1;} // Codage de la ligne
            else if (KB_L == 0xB0) {KB_L = 2;}
            else if (KB_L == 0xD0) {KB_L = 3;}
            else if (KB_L == 0xE0) {KB_L = 4;}
            else {KB_L = 0;};
            if (KB_C == 0x06) {KB_C = 3;} // Codage de la colonne
            else if (KB_C == 0x05) {KB_C = 2;}
            else if (KB_C == 0x03) {KB_C = 1;}
            else {KB_C = 0;};
            if ((KB_C > 0) && (KB_L > 0)) {CarClavier = keys[KB_L-1][KB_C-1];}
        }
    }
    if (CPTSleep)     // Compteur pour suspendre le programme (voir fonction Sleep1ms)
    {
        CPTSleep = CPTSleep-1;
    }
    if (!(ClavierRebond | CPTSleep  | CompteAlarme)) { // S'il n'y a plus rien  compter, on dsactive le timer
        TA0CTL &= ~(MC0 | MC1); // Arrte le timer A0
    }
}


/*
 * Gestion de la pile pour les entres sorties (Last In first out)
 * PileESPUSH : "pousse" la commande  effectuer dans la pile et active les interruptions pour l'mission
 * PileESPOP : donne la commande courante  traiter et l'limine de la pile (ou CmdVide si pile vide)
 * PileESCourant : donne la commande courante  traiter sans la retirer de la pile
 * PileESInit : initialise la pile  vide
 * Index de pile = -1 => Pile vide
 */
void PileESPUSH(uint8_t Commande) {
    if (PileES.Index < MaxPileES) {
        PileES.Index = PileES.Index+1;
        PileES.Commande[PileES.Index] = Commande;
        UCA0IE |= UCTXIE | UCTXCPTIE;
    }
}

char PileESPOP() {
    if (PileES.Index >= 0) {
        PileES.Index = PileES.Index-1;
        return PileES.Commande[PileES.Index+1];
    } else {
        return CmdVide;
    }
}

char PileESCourant() {
    if (PileES.Index >= 0) {
        return PileES.Commande[PileES.Index];
    } else {
        return CmdVide;
    }
}

void PileESInit() {
    PileES.Index = -1;
}


/*
 * Utilise le port 0 qui doit avoir t initialis dans GPIO
 * Initialisation E/S asynchrone 9600bps, 8bits, 1 stop
 * Le programme vrifie si le compteur est connect  une source
 * d'alimentation externe (signal sur PORT_SLEEP).
 * Si oui, il teste si les E/S sont initialises.
 * Si non, il les initialise. Si oui, il ne fait rien.
 * Si pas de signal sur PORT_SLEEP, le programme teste si les E/S
 * sont initialises. Si oui, il les dsactive. Sinon, il ne fait rien
 */

void InitialiseES() {
    if (GPIO_getInputPinValue(PORT_SLEEP, PIN_SLEEP) == GPIO_INPUT_PIN_HIGH) { // Lecteur connect
        if (!(ESInitialise)) { // Si E/S pas encore initialises
            PileESInit();
            LCD_displayAnt(true, true);
            recept.a_traiter = false;       // rien  traiter
            recept.index = 0;
            emet.a_traiter = false;         // rien  traiter
            emet.index = 0;
            // Initialisation des ports d'E/S
            GPIO_setAsPeripheralModuleFunctionInputPin(PORT_TX, PIN_TX, GPIO_PRIMARY_MODULE_FUNCTION);
            GPIO_setAsPeripheralModuleFunctionInputPin(PORT_RX, PIN_RX, GPIO_PRIMARY_MODULE_FUNCTION);
            // Configure UART
            UCA0CTLW0 |= UCSWRST;
            UCA0CTLW0 |= UCSSEL__SMCLK; // 8 bits, 1 stop, pas de parit, SMCLK comme horloge, mode UART, mode asynchrone,
            // Baud Rate calculation
            // 8000000/(16*9600) = 52.083
            // Fractional portion = 0.083
            // User's Guide Table 14-4: UCBRSx = 0x49
            // UCBRFx = int ( (52.083-52)*16) = 1
            UCA0BRW = 52;                             // 8000000/16/9600
            UCA0MCTLW = 0x4900 | UCOS16 | UCBRF_1;
            UCA0CTLW0 &= ~UCSWRST;                    // Initialize eUSCI
            UCA0IE |= UCRXIE;                         // Enable USCI_A0 RX interrupt
            ESInitialise = true;
        }
    } else { // Lecteur non connect
        if (ESInitialise) {
            GPIO_setAsInputPinWithPullDownResistor(PORT_RX, PIN_RX);
            GPIO_setOutputLowOnPin(PORT_TX, PIN_TX);
            GPIO_setAsOutputPin(PORT_TX, PIN_TX);
            LCD_displayAnt(false, false);
            UCA0IE &= 0xFFF0; // Dsactive toutes les interruptions
            UCA0IFG &= 0xFFF0; // pas d'interruptions en cours
            UCA0CTLW0 |= UCSWRST; // Etat Reset pour l'UART
            ESInitialise = false;
        }
    }
}

/*
 * Routine d'interruption E/S srie.
 * Pour des raisons de temps de rponse et d'allocation de buffers, tous les traitements (rception, interprtation...) sont traits
 * dans la routine d'interruption. Normalement, ils ne sont pas trs long et n'interfrent aps avec le reste du programme qui est le plus souvent
 * en LPM3...La partie la plus coteuse en temps est le traitement de la commande P=<paramtres>  cause de l'interprtation
 *
 */
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
  char c;
  switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
  {
    case USCI_NONE: break;
    case USCI_UART_UCRXIFG: // Rception buffer
        c = UCA0RXBUF; // on lit le caractre reu
        if ((c == 0x0A) || (c == 0x0D)) { // si oui, le caractre reu est-il un RC ou LF
            recept.buf[recept.index] = 0x00; // si oui, on termine le buffer avec un caractre null
            lecteur_init();
            c = lecteur_carcourantetinc();
           switch (c)
           {
              case 'D':PileESPUSH(CmdDL1);break;
              case 'F':PileESPUSH(CmdCF);break;
              case 'V':PileESPUSH(CmdVP);break;
              case 'G':PileESPUSH(CmdGD);break;
              default:
                  if (c == 'P') { // P peut tre une commande entrante ou sortante
                      if (lecteur_carcourantetinc() == '=') {
                          if (lecteur_ident("ON")) {
                            AlarmeLed = true;
                          } else {
                            if (lecteur_ident("OFF")) {
                              AlarmeLed = false;
                            }
                          }
                          if (lecteur_carcourantetinc() == ',') {
                            if (lecteur_ident("ON")) {
                              AlarmeBuz = true;
                            } else {
                              if (lecteur_ident("OFF")) {
                                AlarmeBuz = false;
                              }
                            }
                          }
                          if (lecteur_carcourantetinc() == ',') {
                              lecteur_ident("PM"); // On doit toujours avoir PM !
                          }
                          if (lecteur_carcourantetinc() == ',') {
                            lecteur_int(&SeuilAlarme);
                          }
                      }
                      PileESPUSH(CmdPC);
                  }
              break;
            }
            UCA0IE |= UCTXIE | UCTXCPTIE;
            recept.index = 0;
        } else { // sinon (rien  traiter, pas de caractre null)
            if (recept.index < BUF_MAX) { // on vrifie si on atteint la fin du buffer
                recept.buf[recept.index] = c; // S'il reste de la place, on stocke le caractre
                recept.index = recept.index+1; // et on incrmente l'index dans le buffer
            }
        }
      break;

      case USCI_UART_UCTXIFG:
        if (!(emet.a_traiter)) {
            emet.index = 0;
            emet.a_traiter = true;
            switch (PileESPOP()) {
            case CmdGD:
                strcpy(emet.buf,"ID=\x0");
                Uint16Ascii(Vbat,emet.buf);
                strcat(emet.buf,",");
                Uint16Ascii(NbMoyHeures,emet.buf);
                strcat(emet.buf,",");
                if (MoyHeures.Plein) {Uint16Ascii(NbMoyHeures,emet.buf);} else {Uint16Ascii(MoyHeures.Index+1,emet.buf);}
                strcat(emet.buf,",");
                Uint16Ascii(NbJours,emet.buf);
            break;
            case CmdPC:
                strcpy(emet.buf,"PC=");
                if (AlarmeLed) {strcat(emet.buf,"ON");} else {strcat(emet.buf,"OFF");}
                strcat(emet.buf,",");
                if (AlarmeBuz) {strcat(emet.buf,"ON");} else {strcat(emet.buf,"OFF");}
                strcat(emet.buf,",PM,"); // toujours en CPM pour PCGM8
                Uint16Ascii(SeuilAlarme,emet.buf);
            break;
            case CmdCF:
                strcpy(emet.buf,T_FacteurConversion);
            break;
            case CmdVP:
                strcpy(emet.buf,T_Invite);
            break;
            case CmdDL3:
                strcpy(emet.buf,T_DL);
                strcat(emet.buf,T_End);
            break;
            case CmdCM :
                strcpy(emet.buf,"CM=");
                Uint16Ascii(CountParMinute,emet.buf);
                break;
            case CmdDL1 :
                if (MoyHeures.Index == -1) {
                    strcpy(emet.buf,T_DL);
                    strcat(emet.buf,T_NoData);
                } else {MoyHeures.IndexES = MoyHeures.Index;}
            case CmdDL2 :
                if (MoyHeures.Index >= 0) {
                    strcpy(emet.buf,T_DL);
                    Uint16Ascii(MoyHeures.Buf[MoyHeures.IndexES],emet.buf);
                    MoyHeures.IndexES = MoyHeures.IndexES-1;
                    if (MoyHeures.IndexES < 0) {
                        if (MoyHeures.Plein) {MoyHeures.IndexES = NbMoyHeures-1;}
                    }
                    if ((MoyHeures.IndexES >= 0) && (MoyHeures.IndexES != MoyHeures.Index)) {PileESPUSH(CmdDL2);} else {PileESPUSH(CmdDL3);}
                }
            break;
            default:
                emet.a_traiter = false;
                PileESInit();
            case CmdVide:
                UCA0IE &= ~(UCTXIE); // on dsactive l'interruption sur buffer mission vide
            break;
            }
            if (emet.a_traiter) {UCA0IE |= UCTXIE | UCTXCPTIE;}
        }
        if (emet.a_traiter) { // s'il y a quelque chose  mettre
            c = emet.buf[emet.index]; // caractre  mettre dans c
            if (c == 0x00) { // si fin de chaine, alors
                UCA0TXBUF = 0x0D; // met RC
                emet.a_traiter = false; // indique qu'il n'y  plus rien  mettre
            } else { // sinon,
                UCA0TXBUF = c; // met le caractre courant
                emet.index = emet.index+1; // passe au caractre suivant. Risque de dbordement si le buffer n'a pas de null !
            }
        }

      break;
    case USCI_UART_UCSTTIFG: break;
    case USCI_UART_UCTXCPTIFG: // registre  dcallage vide
        if (!(emet.a_traiter) && (PileESCourant() == CmdVide)) { // si on n'a plus rien  mettre
            UCA0IE &= ~(UCTXIE | UCTXCPTIE); // alors, on dsactive toutes les interruptions en mission
        } // sinon, on ne fait rien
        break;
    default: break;
  }
}


/*
 * Donne la tension de la batterie en centivolts.
 * Ex: 367 = 3,67V
 * VBat = tension au niveau de la batterie. Mis  jour automatiquement toutes les 24h ou  la demande. Variable globale
 */
void TensionBatterie()
{

   uint32_t adcResult;                      // Pour lecture ADC

    ADC12MCTL0 = ADC12INCH3 | ADC12VRSEL0; // A8 = P9.0 est l'entre analogique (0x0008). 01000b = If ADC12DIF = 0: A8; If ADC12DIF = 1: Ain+ = A8, Ain- = A9
                             // VR+ = VRef et VR- = AVSS  => Mem1 ???


    // Mesure de la tension aux bornes de la batterie

    GPIO_setOutputHighOnPin(PORT_BAT_EN, PIN_BAT_EN); // active la mesure de la batterie
    SleepMS(10); // on attend que a se stabilise

    ADC12CTL0 &= ~(ADC12ON + ADC12ENC + ADC12SC);    // Dsactive le convertisseur
    ADC12IER1 &= 0;                             // Invalide toutes les interruptions
    ADC12IER2 &= 0;                             // Invalide toutes les interruptions
    ADC12IFGR0 &= 0;                            // plus d'interruptions en cours
    ADC12IFGR1 &= 0;                            // plus d'interruptions en cours
    ADC12IFGR2 &= 0;                            // plus d'interruptions en cours
    ADC12MCTL0 = ADC12VRSEL_4 | ADC12INCH_8;      //  VeREF+, VR- = AVSS et P9.0 = A8
    ADC12CTL1 = ADC12SHP;                       // Use sampling timer
    ADC12CTL2 = ADC12RES1;                      //12 bits rsolution

    while(REFCTL0 & REFGENBUSY);              // Si le gnrateur de rfrence est occup, attente
    REFCTL0 |= REFVSEL_1 | REFON;             // Slection de la rfrence 2V et rfrence interne on
    while(!(REFCTL0 & REFGENRDY));            // Attente stabilisation rfrence

    ADC12CTL0 |= ADC12ON | ADC12SHT0_2;
    ADC12CTL0 |= ADC12ENC; // Active conversion et dmarre conversion
    ADC12CTL0 |= ADC12SC;

    while (!(ADC12IFGR0 & BIT0));{SleepMS(2);}
    adcResult = ADC12MEM8;
    ADC12IFGR0 &= ~BIT0;
    //
    // Constante :
    //    4159 : tension mesure aux bornes de la batterie
    //    1112 : valeur affiche par le convertisseur (ADC12MEM8)
    if (adcResult > 0) {adcResult = (adcResult * (unsigned long)4159) / (unsigned long)1112;} else {adcResult = 0;}
    Vbat = adcResult/10;
    GPIO_setOutputLowOnPin(PORT_BAT_EN, PIN_BAT_EN); // desactive la mesure de la batterie
    ADC12CTL0 &= ~ADC12ON;
    REFCTL0 |= REFVSEL_1 ;                      // Rfrence interne plus dispo ?
}





